-----------------------------------------------------------------------------
-- Project Name: Robot unit
-- File Name:    change_detect.vhd
-- Description:  Detect changes on input vector and set output to one period
-- Author:       Jakub Podivinsky <xpodiv01@stud.fit.vutbr.cz> 
-- Date:         15.4.2013 
-- --------------------------------------------------------------------------

library ieee;
use ieee.std_logic_1164.all;
-- ==========================================================================
--                              ENTITY DECLARATION
-- ==========================================================================
entity CHANGE_ENT is
   generic
   (
      DATA_WIDTH  : integer := 4      
   );
   
   port
   (
      CLK            : in  std_logic;
      RST            : in  std_logic; 
      
      -- ----------------- INPUT INTERFACE ----------------------- 
      DATA_IN        : in std_logic_vector(DATA_WIDTH-1 downto 0);
              
      -- ----------------- OUTPUT INTERFACE ----------------------
      OUTPUT         : out  std_logic
 
   );   
end entity;

-- ==========================================================================
--                           ARCHITECTURE DESCRIPTION
-- ==========================================================================
architecture CHANGE_ARCH of CHANGE_ENT is

-- ==========================================================================
--                                      TYPES
-- ==========================================================================

-- ==========================================================================
--                                    CONSTANTS
-- ==========================================================================

-- ==========================================================================
--                                     SIGNALS
-- ==========================================================================
-- data signals
signal reg_IN                 : std_logic_vector(DATA_WIDTH-1 downto 0);

-- control signals
signal sig_change_val         : std_logic;



begin
   OUTPUT <= sig_change_val;
   
   -- detect event on input data => enable evaluating
   change_register : process (CLK, DATA_IN, reg_IN)
   begin
      if( rising_edge( CLK ) ) then
         if (RST = '1') then
            sig_change_val <= '0';
         elsif (DATA_IN /= reg_IN) then
            sig_change_val <= '1';
         else
            sig_change_val <= '0';
         end if;
      end if;
   end process;

   -- save old data to detect change
   old_registers : process (CLK, RST, DATA_IN)
   begin
      if( rising_edge( CLK ) ) then
         reg_IN <= DATA_IN;
      end if;
   end process;     
end architecture;
