-----------------------------------------------------------------------------
-- Project Name: Robot unit
-- File Name:    bdu_fsm_tl.vhd
-- Description:  Top level of bdu and bdu_fsm for testing funcionallity
-- Author:       Jakub Podivinsky <xpodiv01@stud.fit.vutbr.cz> 
-- Date:         28.1.2013 
-- --------------------------------------------------------------------------

library ieee;
use ieee.std_logic_1164.all;
--use ieee.numeric_std.all;
--use ieee.std_logic_unsigned.all;
-- ==========================================================================
--                              ENTITY DECLARATION
-- ==========================================================================
entity BDU_FSM_TL_ENT is
   generic
   (
      DATA_WIDTH  : integer := 16      
   );
   
   port
   (
      CLK            : in  std_logic;
      RST            : in  std_logic; 
      
      -- ----------------- INPUT DATA --------------------------------- 
      S_0            : in std_logic_vector(DATA_WIDTH-1 downto 0);
      S_1            : in std_logic_vector(DATA_WIDTH-1 downto 0);
      S_2            : in std_logic_vector(DATA_WIDTH-1 downto 0);
      S_3            : in std_logic_vector(DATA_WIDTH-1 downto 0);

      NOT_CYC_WRITE  : in std_logic;
              
      -- ----------------- OUTPUT DATA ---------------------------
      BAR            : out std_logic_vector(DATA_WIDTH-1 downto 0)
 
   );   
end entity;

-- ==========================================================================
--                           ARCHITECTURE DESCRIPTION
-- ==========================================================================
architecture BDU_FSM_TL_ARCH of BDU_FSM_TL_ENT is

-- ==========================================================================
--                                      TYPES
-- ==========================================================================

-- ==========================================================================
--                                    CONSTANTS
-- ==========================================================================

-- ==========================================================================
--                                     SIGNALS
-- ==========================================================================

-- data signals
signal sig_S_0                : std_logic_vector(DATA_WIDTH-1 downto 0);
signal sig_S_1                : std_logic_vector(DATA_WIDTH-1 downto 0);
signal sig_S_2                : std_logic_vector(DATA_WIDTH-1 downto 0);
signal sig_S_3                : std_logic_vector(DATA_WIDTH-1 downto 0);
signal sig_BAR                : std_logic_vector(DATA_WIDTH-1 downto 0);

-- control signals
signal sig_act                : std_logic;
signal sig_vld                : std_logic;


begin  
   bdu : entity work.bdu_ent
   generic map(
      DATA_WIDTH  => DATA_WIDTH
   )   
   port map(
      CLK            => CLK,
      RST            => RST,
      
      -- input interface
      S_0            => sig_S_0, 
      S_1            => sig_S_1, 
      S_2            => sig_S_2, 
      S_3            => sig_S_3, 
           
      -- output interface      
      OUTPUT         => sig_BAR,
      
      -- control interface
      ACT            => sig_act,
      VLD            => sig_vld
   );

   bdu_fsm : entity work.bdu_fsm_ent
   generic map(
      DATA_WIDTH  => DATA_WIDTH      
   )   
   port map(
      CLK            => CLK,
      RST            => RST, 
      
      -- ----------------- INPUT/OUTPUT DATA --------------------- 
      S_0            => S_0, 
      S_1            => S_1,
      S_2            => S_2,
      S_3            => S_3,
      NOT_CYC_WRITE  => NOT_CYC_WRITE,
      BAR            => BAR,
      -- ----------------- BDU DATA ------------------------------ 
      S_0_BDU        => sig_S_0,
      S_1_BDU        => sig_S_1,
      S_2_BDU        => sig_S_2, 
      S_3_BDU        => sig_S_3,
           
      -- output interface      
      BAR_BDU        => sig_BAR,
      
      -- control interface
      ACT_DBU        => sig_act,
      VLD_BDU        => sig_vld
 
   );   
end architecture;
