-----------------------------------------------------------------------------
-- Project Name: ROBOT
-- File Name:    ecu_fsm.vhd
-- Description:  Controll getting data from PFU and move by using block MOVE 
-- Author:       Jakub Podivinsky <xpodiv01@stud.fit.vutbr.cz> 
-- Date:         25.2.2013 
-- --------------------------------------------------------------------------

library ieee;
use ieee.std_logic_1164.all;
--use ieee.std_logic_arith.all;
--use ieee.std_logic_signed.all;

-- ==========================================================================
--                              ENTITY DECLARATION
-- ==========================================================================
entity ECU_FSM_ENT is
   generic
   (
      DATA_WIDTH     : integer := 16     
   );
   
   port
   (
      CLK            : in std_logic;
      RST            : in std_logic;
      -- ----------------- FIFO INPUT --------------------------------
      X              : in std_logic_vector(DATA_WIDTH - 1 downto 0);
      Y              : in std_logic_vector(DATA_WIDTH - 1 downto 0);
      
      FIFO_ACT       : out std_logic;
      FIFO_VLD       : in std_logic;
      -- ----------------- MOVE OUTPUT -------------------------
      L              : out std_logic;
      R              : out std_logic;
      T              : out std_logic;
      B              : out std_logic;
      
      ACT            : out std_logic;
      VLD            : in std_logic
   );
   
end entity;

-- ==========================================================================
--                           ARCHITECTURE DESCRIPTION
-- ==========================================================================
architecture ECU_FSM_ARCH of ECU_FSM_ENT is

-- ==========================================================================
--                                      TYPES
-- ==========================================================================
type state_type is (init_state, read_state, eval_dir_state, move_state, wait_state); 

-- ==========================================================================
--                                    CONSTANTS
-- ==========================================================================

-- ==========================================================================
--                                     SIGNALS
-- ==========================================================================
-- FSM signals 
signal state_reg, state_next: state_type;

-- signals
signal sig_dir_we              : std_logic;


begin
-- ------------------------- FSM --------------------------------------------   
   -- state logic
   fsm_state_logic : process (CLK)
   begin
     if (rising_edge(CLK)) then
        if (RST = '1') then
          state_reg <= init_state;
        else
          state_reg <= state_next;
        end if;   
     end if;   
   end process;
   
   -- next state logic
   fsm_next_state_logic: process( state_reg, FIFO_VLD, VLD) --stavova logika
   begin
      case state_reg is
         when init_state =>
            if (FIFO_VLD = '1') then
               FIFO_ACT    <= '1';
               ACT         <= '0';
               sig_dir_we  <= '0';
               state_next  <= read_state;
            else
               FIFO_ACT    <= '1';
               ACT         <= '0';
               sig_dir_we  <= '0';
               state_next  <= init_state;
            end if;
            
         when read_state =>
            FIFO_ACT    <= '0';
            ACT         <= '0';
            sig_dir_we  <= '0';
            state_next  <= eval_dir_state;
            
         when eval_dir_state =>
            if (FIFO_VLD = '1') then
               FIFO_ACT    <= '1';
               ACT         <= '0';
               sig_dir_we  <= '0';
               state_next  <= wait_state;
            else
               FIFO_ACT    <= '1';
               ACT         <= '0';
               sig_dir_we  <= '0';
               state_next  <= eval_dir_state;
            end if;
            
         when wait_state =>
            FIFO_ACT    <= '0';
            ACT         <= '0';
            sig_dir_we  <= '1';
            state_next  <= move_state;   
            
         when move_state =>
            if (VLD = '1') then
               FIFO_ACT    <= '0';
               ACT         <= '1';
               sig_dir_we  <= '0';
               state_next  <= read_state;
            else
               FIFO_ACT    <= '0';
               ACT         <= '1';
               sig_dir_we  <= '0';
               state_next  <= move_state;
            end if;
            
         when others => 
            FIFO_ACT    <= '0';
            ACT         <= '0';
            sig_dir_we  <= '0';
            state_next  <= init_state;
      end case;
   end process;  
 
   -- evaluate move direction
   direction : entity work.DIR_ENT
   generic map(
      DATA_WIDTH  => DATA_WIDTH
   )   
   port map(
      CLK            => CLK,
      RST            => RST,
      L              => L,
      R              => R,
      T              => T,
      B              => B,
      X              => X,
      Y              => Y,
      WE             => sig_dir_we 
   );   
   
end architecture;