library ieee;
use ieee.std_logic_1164.all;

entity testbench is
end entity testbench;

architecture tb_arch of testbench is
  -- perioda hodin 
  constant clk_period   : time := 20 ns;
  constant reset_time   : time := 100 ns;
  constant data_width   : integer := 4;
  constant addr_width   : integer := 13;
    
  -- prepajacie signaly
  signal doa            : std_logic_vector(data_width-1 downto 0);  -- 32-bit A port data output
  signal dob            : std_logic_vector(data_width-1 downto 0); -- 32-bit B port data output
  signal addra          : std_logic_vector(addr_width-1 downto 0);  -- 16-bit A port address input
  signal addrb          : std_logic_vector(addr_width-1 downto 0); -- 16-bit B port address input
  signal clka           : std_logic; -- 1-bit A port clock input
  signal clkb           : std_logic; -- 1 bit B port clock input
  signal dia            : std_logic_vector(data_width-1 downto 0); -- 32-bit A port data input
  signal dib            : std_logic_vector(data_width-1 downto 0); -- 32-bit B port data input
  signal ena            : std_logic; -- 1-bit A port enable input
  signal enb            : std_logic; -- 1-bit B port enable input
  signal ssra           : std_logic; -- 1-bit A port set/reset input
  signal ssrb           : std_logic; -- 1-bit B port set/reset input
  signal wea            : std_logic; -- 4-bit A port write enable input
  signal web            : std_logic; -- 4-bit B port write enable input

begin
  -- instancia testovanej komponenty
  UUT : entity work.BRAM_ENT
  port map( DOA            => doa,
            DOB            => dob,
            ADDRA          => addra,
            ADDRB          => addrb,
            CLKA           => clka,
            CLKB           => clkb,
            DIA            => dia,
            DIB            => dib,
            ENA            => ena,
            ENB            => enb,
            SSRA           => ssra,
            SSRB           => ssrb,
            WEA            => wea,
            WEB            => web
  );

  -- generovanie hodinoveho signalu 
  clk_gen :  process 
  begin  
    clka     <='1';
    clkb     <='1';
    wait for clk_period/2;
    clka     <='0';
    clkb     <='0';
    wait for clk_period/2;
  end process clk_gen;
  
  resetgen: process
   begin
      ssra   <= '0';
      ssrb   <= '0';
      wait for reset_time;
      ssra   <= '0';
      ssrb   <= '0';
      wait for 20*clk_period;
      ssra   <= '0';
      ssrb   <= '0';
      wait;
   end process;

  -- generovanie vstupov
  behaviour : process
  begin
    ena    <= '0';
    wea    <= '0';
    addra  <= "0000000000001";
    wait for clk_period;
    dia <= "0001"; 
    wait for clk_period;
    wea    <= '1';
    wait for clk_period;
    addra  <= "0000000000001";   
    wea    <= '0';
    ena    <= '1';
    wait for clk_period;
    addra  <= "0000000000001";
    
    wait for 10*clk_period;
    
    wea    <= '0';
    addra  <= "1111111111111";
    wait for clk_period;
    dia <= "0001"; 
    wait for clk_period;
    wea    <= '1';
    wait for 3*clk_period;
    addra  <= "1111111111111";   
    wea    <= '0';
    wait for clk_period;
    addra  <= "1111111111111";
   
    
    wait;
  end process behaviour; 
   
end architecture tb_arch;
