library ieee;
use ieee.std_logic_1164.all;
use ieee.numeric_std.all;

entity testbench is
end entity testbench;

architecture tb_arch of testbench is
  -- perioda hodin 
  constant clk_period   : time := 20 ns;
  constant reset_time   : time := 100 ns;
  constant data_width   : integer := 16;
  constant addr_width   : integer := 13;
  
  -- prepajacie signaly
  signal clk         : std_logic;
  signal rst         : std_logic;
  
  signal addr_x      : std_logic_vector(data_width-1 downto 0);
  signal addr_y      : std_logic_vector(data_width-1 downto 0);
  signal data_out    : std_logic_vector(3 downto 0);
  signal data_in     : std_logic_vector(3 downto 0);
  signal rd_wr       : std_logic;
  signal act         : std_logic;
  signal vld         : std_logic;
  
  signal addr        : std_logic_vector(addr_width-1 downto 0);
  signal di          : std_logic_vector(3 downto 0);
  signal do          : std_logic_vector(3 downto 0);
  signal we          : std_logic;
  signal en          : std_logic;
  signal mclk        : std_logic;
  signal ssr         : std_logic;
      

begin
  -- instancia testovanej komponenty
  UUT : entity work.MC_ENT
  port map( CLK         => clk,
            RST         => rst,
            ADDR_X      => addr_x,
            ADDR_Y      => addr_y,
            DATA_OUT    => data_out,
            DATA_IN     => data_in,
            RD_WR       => rd_wr, 
            ACT         => act,
            VLD         => vld,
            ADDR        => addr,
            DI          => di,
            DO          => do,
            WE          => we,
            EN          => en,
            MCLK        => mclk,
            SSR         => ssr            
  );

  -- generovanie hodinoveho signalu 
  clk_gen :  process 
  begin  
    clk     <='1';
    wait for clk_period/2;
    clk     <='0';
    wait for clk_period/2;
  end process clk_gen;
  
  resetgen: process
   begin
      rst   <= '1';
      wait for reset_time;
      rst   <= '0';
      wait for 20*clk_period;
      rst   <= '0';
      wait;
   end process;

  -- generovanie vstupov
  behaviour : process
  begin
    addr_x     <= std_logic_vector(to_unsigned(23, DATA_WIDTH));
    addr_y     <= std_logic_vector(to_unsigned(1, DATA_WIDTH));
    data_in    <= "1111";  
  
    act        <= '0';
    rd_wr      <= '0';
    wait for 2*reset_time;
    act        <= '1';
    wait for 2*clk_period;
    act        <= '0';
    
    
    wait for clk_period;
    rd_wr      <= '1';
    act        <= '1';
    do         <= "0101";
    wait for 2*clk_period;
    act        <= '0';


    wait;
  end process behaviour; 
   
end architecture tb_arch;
