library ieee;
use ieee.std_logic_1164.all;
use ieee.numeric_std.all;

entity testbench is
end entity testbench;

architecture tb_arch of testbench is
  -- perioda hodin 
  constant clk_period   : time := 20 ns;
  constant reset_time   : time := 100 ns;
  constant data_width   : integer := 16;
  
  -- prepajacie signaly
  signal clk            : std_logic;
  signal rst            : std_logic;
      
 -- ----------------- input interface ----------------------- 
  signal wb_x           : std_logic_vector(data_width-1 downto 0);
  signal wb_y           : std_logic_vector(data_width-1 downto 0);
  signal wb_bar         : std_logic_vector(data_width-1 downto 0);
  signal wb_act         : std_logic;
  signal wb_vld         : std_logic; 
 -- ----------------- output interface ----------------------
  signal mu_x           : std_logic_vector(data_width-1 downto 0);
  signal mu_y           : std_logic_vector(data_width-1 downto 0);
  signal mu_bar         : std_logic_vector(3 downto 0);
  signal mu_act         : std_logic;
  signal mu_vld         : std_logic; 
      

begin
  -- instancia testovanej komponenty
  UUT : entity work.MU_FSM_ENT
  port map( CLK            => clk,
            RST            => rst,
            -- ----------------- INPUT INTERFACE ----------------------- 
            WB_X           => wb_x,
            WB_Y           => wb_y,
            WB_BAR         => wb_bar,
            WB_ACT         => wb_act,
            WB_VLD         => wb_vld,
                    
            -- ----------------- OUTPUT INTERFACE ----------------------
            MU_X           => mu_x,
            MU_Y           => mu_y,
            MU_BAR         => mu_bar,
            MU_ACT         => mu_act,
            MU_VLD         => mu_vld    
             
  );

  -- generovanie hodinoveho signalu 
  clk_gen :  process 
  begin  
    clk     <='1';
    wait for clk_period/2;
    clk     <='0';
    wait for clk_period/2;
  end process clk_gen;
  
  resetgen: process
   begin
      rst   <= '1';
      wait for reset_time;
      rst   <= '0';
      wait for 20*clk_period;
      rst   <= '0';
      wait;
   end process;

  -- generovanie vstupov
  behaviour : process
  begin
    WB_VLD <= '0';
    MU_VLD <= '0';
    wait for reset_time;
    wait for clk_period;
    wait for 3*clk_period;
    WB_X <= "1111000011110000";
    wait for 2*clk_period;
    WB_Y <= "0111100001111000";
    wait for 2*clk_period;
    WB_BAR <= "0000000000001001";
    WB_VLD <= '1';
    wait for clk_period;
    WB_VLD <= '0';
    wait for 4*clk_period;
    MU_VLD <= '1';
    wait for clk_period;
    MU_VLD <= '0';
    
    wait for clk_period;
    wait for 3*clk_period;
    WB_X <= "1111000011110000";
    wait for 2*clk_period;
    WB_Y <= "0111100001111000";
    wait for 2*clk_period;
    WB_BAR <= "0000000000001001";
    WB_VLD <= '1';
    wait for clk_period;
    WB_VLD <= '0';
    
    wait;
  end process behaviour; 
   
end architecture tb_arch;
