library ieee;
use ieee.std_logic_1164.all;
use ieee.numeric_std.all;
use ieee.std_logic_arith.all;

entity testbench is
end entity testbench;

architecture tb_arch of testbench is
  -- perioda hodin 
  constant clk_period   : time := 20 ns;
  constant reset_time   : time := 100 ns;
  constant data_width   : integer := 16;
  
  -- prepajacie signaly
  signal clk            : std_logic;
  signal rst            : std_logic;
      
      -- ----------------- pfu_wb interface --------------------- 
  signal x_wb           : std_logic_vector(data_width-1 downto 0);
  signal y_wb           : std_logic_vector(data_width-1 downto 0);
  signal act_wb         : std_logic;
  signal vld_wb         : std_logic;
      
      -- ----------------- ecu interface ------------------------ 
  signal x_ecu          : std_logic_vector(data_width-1 downto 0);
  signal y_ecu          : std_logic_vector(data_width-1 downto 0);
  signal act_ecu        : std_logic;
  signal vld_ecu        : std_logic;
      
      -- ----------------- mmu interface ------------------------ 
  signal x_mmu          : std_logic_vector(data_width-1 downto 0);
  signal y_mmu          : std_logic_vector(data_width-1 downto 0);
  signal act_mmu        : std_logic;
  signal vld_mmu        : std_logic;
  signal rd_wr_mmu      : std_logic;
  signal data_in_mmu    : std_logic_vector(3 downto 0);
      

begin
  -- instancia testovanej komponenty
  UUT : entity work.PFU_ENT
  port map( CLK            => clk,
            RST            => rst,
            -- ----------------- PFU_WB INTERFACE --------------------- 
            X_WB           => x_wb,
            Y_WB           => y_wb,
            ACT_WB         => act_wb,
            VLD_WB         => vld_wb,
            
            -- ----------------- ECU INTERFACE ------------------------ 
            X_ECU          => x_ecu,
            Y_ECU          => y_ecu,
            ACT_ECU        => act_ecu,
            VLD_ECU        => vld_ecu,
            
            -- ----------------- MMU INTERFACE ------------------------ 
            X_MMU          => x_mmu,
            Y_MMU          => y_mmu,
            ACT_MMU        => act_mmu,
            VLD_MMU        => vld_mmu,
            RD_WR_MMU      => rd_wr_mmu,
            DATA_IN_MMU    => data_in_mmu  
  );

  -- generovanie hodinoveho signalu 
  clk_gen :  process 
  begin  
    clk     <='1';
    wait for clk_period/2;
    clk     <='0';
    wait for clk_period/2;
  end process clk_gen;
  
  resetgen: process
   begin
      rst   <= '1';
      wait for 2*reset_time;
      rst   <= '0';
      wait for 20*clk_period;
      rst   <= '0';
      wait;
   end process;

  -- generovanie vstupov
  behaviour : process
  begin
    vld_wb   <= '0';
    vld_mmu  <= '0';
    vld_ecu  <= '0';
    
    --wait for reset_time;
    --wait for clk_period;
    x_wb    <= conv_std_logic_vector(1, DATA_WIDTH);
    y_wb    <= conv_std_logic_vector(6, DATA_WIDTH);
    wait for 2*reset_time;
    wait for 2*clk_period;
    vld_wb  <= '1';
    wait for clk_period;
    vld_wb  <= '0';
    
    wait for 2*clk_period;
    vld_ecu <= '1';
    wait for clk_period;
    vld_ecu <= '0';
    
    wait for 4*clk_period;
    data_in_mmu <= "0000";
    vld_mmu <= '1';
    wait for clk_period;
    vld_mmu <= '0';
    
    wait for 4*clk_period;
    data_in_mmu <= "0000";
    vld_mmu <= '1';
    wait for clk_period;
    vld_mmu <= '0';
    
    
    wait for 4*clk_period;
    data_in_mmu <= "0010";
    vld_mmu <= '1';
    wait for clk_period;
    vld_mmu <= '0';
    
    wait for 4*clk_period;
    vld_ecu <= '1';
    wait for clk_period;
    vld_ecu <= '0';
    
    wait for 4*clk_period;
    data_in_mmu <= "0011";
    vld_mmu <= '1';
    wait for clk_period;
    vld_mmu <= '0';
    
     wait for 4*clk_period;
    data_in_mmu <= "0011";
    vld_mmu <= '1';
    wait for clk_period;
    vld_mmu <= '0';
    
    wait for 4*clk_period;
    data_in_mmu <= "0010";
    vld_mmu <= '1';
    wait for clk_period;
    vld_mmu <= '0';
    
    wait for 4*clk_period;
    vld_ecu <= '1';
    wait for clk_period;
    vld_ecu <= '0';
    
    wait for 4*clk_period;
    data_in_mmu <= "0000";
    vld_mmu <= '1';
    wait for clk_period;
    vld_mmu <= '0';
    
    wait for 4*clk_period;
    data_in_mmu <= "0000";
    vld_mmu <= '1';
    wait for clk_period;
    vld_mmu <= '0';
    
    wait for 4*clk_period;
    data_in_mmu <= "0011";
    vld_mmu <= '1';
    wait for clk_period;
    vld_mmu <= '0';
    
    wait for 4*clk_period;
    data_in_mmu <= "0010";
    vld_mmu <= '1';
    wait for clk_period;
    vld_mmu <= '0';
    
    wait for 4*clk_period;
    vld_ecu <= '1';
    wait for clk_period;
    vld_ecu <= '0';
    
    wait for 4*clk_period;
    data_in_mmu <= "0011";
    vld_mmu <= '1';
    wait for clk_period;
    vld_mmu <= '0';
    
    wait for 4*clk_period;
    data_in_mmu <= "0010";
    vld_mmu <= '1';
    wait for clk_period;
    vld_mmu <= '0';
    
    wait for 4*clk_period;
    vld_ecu <= '1';
    wait for clk_period;
    vld_ecu <= '0';
    
    wait for 4*clk_period;
    data_in_mmu <= "0010";
    vld_mmu <= '1';
    wait for clk_period;
    vld_mmu <= '0';
    
    wait for 4*clk_period;
    vld_ecu <= '1';
    wait for clk_period;
    vld_ecu <= '0';
    
    wait for 4*clk_period;
    data_in_mmu <= "0011";
    vld_mmu <= '1';
    wait for clk_period;
    vld_mmu <= '0';
    
    wait for 4*clk_period;
    data_in_mmu <= "0010";
    vld_mmu <= '1';
    wait for clk_period;
    vld_mmu <= '0';
    
    wait for 4*clk_period;
    vld_ecu <= '1';
    wait for clk_period;
    vld_ecu <= '0';
    
    wait for 4*clk_period;
    data_in_mmu <= "0010";
    vld_mmu <= '1';
    wait for clk_period;
    vld_mmu <= '0';
    
    wait for 4*clk_period;
    vld_ecu <= '1';
    wait for clk_period;
    vld_ecu <= '0';
    
    wait;
  end process behaviour; 
   
end architecture tb_arch;
