-----------------------------------------------------------------------------
-- Project Name: ROBOT
-- File Name:    top_level.vhd
-- Description:  Propojuje vysilac(master), Wishbone sbernici a prijimac(slave). 
-- Author:       JP
-- Date:         9.11.2012 
-- --------------------------------------------------------------------------

library ieee;
use ieee.std_logic_1164.all;

entity ROBOT_CONTROLLER_ENT is
   generic
   (
      DATA_WIDTH     : integer := 16;  
      ADDR_WIDTH     : integer := 8;
      OFFSET_WIDTH   : integer := 4;
      QUEUE_SIZE     : integer := 15      
   );
   port(
      CLK             : in std_logic;
      RST             : in std_logic; 
      
      DIST_A          : in std_logic_vector(DATA_WIDTH-1 downto 0);
      DIST_B          : in std_logic_vector(DATA_WIDTH-1 downto 0);
      DIST_C          : in std_logic_vector(DATA_WIDTH-1 downto 0);
      S_0             : in std_logic_vector(DATA_WIDTH-1 downto 0);
      S_1             : in std_logic_vector(DATA_WIDTH-1 downto 0);
      S_2             : in std_logic_vector(DATA_WIDTH-1 downto 0);
      S_3             : in std_logic_vector(DATA_WIDTH-1 downto 0);
      DATA_VLD        : in  std_logic; 
      X_SPEED         : out std_logic_vector(DATA_WIDTH - 1 downto 0);
      Y_SPEED         : out std_logic_vector(DATA_WIDTH - 1 downto 0)
      
      -- =================================================================
      -- PFU
      -- =================================================================
      -- PFU_READ_MASTER
      --PFU_ACT           : in std_logic;
      --PFU_VLD           : out std_logic;
      --PFU_X             : out std_logic_vector( DATA_WIDTH-1 downto 0 );
      --PFU_Y             : out std_logic_vector( DATA_WIDTH-1 downto 0 );
      
      -- =================================================================
      -- PFU MEM
      -- =================================================================
      -- ----------------- PORT B ---------------------------       
      --ADDR_X            : in std_logic_vector(DATA_WIDTH-1 downto 0);
      --ADDR_Y            : in std_logic_vector(DATA_WIDTH-1 downto 0);
      --DATA_OUT          : out std_logic_vector(3 downto 0)
      
   );
end entity;

architecture ROBOT_CONTROLLER_ARCH of ROBOT_CONTROLLER_ENT is
  -- connect blocks to wishbone bus
  signal sif_dat_i               : std_logic_vector(15 downto 0);
  signal sif_ack_i               : std_logic;
  signal sif_dat_o               : std_logic_vector(15 downto 0);
  signal sif_we_o                : std_logic;
  signal sif_sel_o               : std_logic_vector(1 downto 0);
  signal sif_adr_o               : std_logic_vector(7 downto 0);
  signal sif_cyc_o               : std_logic;
  signal sif_stb_o               : std_logic;
  
  signal mu_dat_i                : std_logic_vector(15 downto 0);
  signal mu_ack_i                : std_logic;
  signal mu_dat_o                : std_logic_vector(15 downto 0);
  signal mu_we_o                 : std_logic;
  signal mu_sel_o                : std_logic_vector(1 downto 0);
  signal mu_adr_o                : std_logic_vector(7 downto 0);
  signal mu_cyc_o                : std_logic;
  signal mu_stb_o                : std_logic;

  signal pfu_dat_i               : std_logic_vector(15 downto 0);
  signal pfu_ack_i               : std_logic;
  signal pfu_dat_o               : std_logic_vector(15 downto 0);
  signal pfu_we_o                : std_logic;
  signal pfu_sel_o               : std_logic_vector(1 downto 0);
  signal pfu_adr_o               : std_logic_vector(7 downto 0);
  signal pfu_cyc_o               : std_logic;
  signal pfu_stb_o               : std_logic;
  
  signal peu_dat_o               : std_logic_vector(15 downto 0);
  signal peu_ack_o               : std_logic;
  signal peu_dat_i               : std_logic_vector(15 downto 0);
  signal peu_we_i                : std_logic;
  signal peu_sel_i               : std_logic_vector(1 downto 0);
  signal peu_adr_i               : std_logic_vector(3 downto 0);
  signal peu_cyc_i               : std_logic;
  signal peu_stb_i               : std_logic;
  
  signal bdu_dat_o               : std_logic_vector(15 downto 0);
  signal bdu_ack_o               : std_logic;
  signal bdu_dat_i               : std_logic_vector(15 downto 0);
  signal bdu_we_i                : std_logic;
  signal bdu_sel_i               : std_logic_vector(1 downto 0);
  signal bdu_adr_i               : std_logic_vector(3 downto 0);
  signal bdu_cyc_i               : std_logic;
  signal bdu_stb_i               : std_logic;
  
  --others signals
  -- SIF
  signal sif_fsm_act             : std_logic;
  signal sif_fsm_vld             : std_logic;
  signal sif_fsm_a               : std_logic_vector( DATA_WIDTH-1 downto 0 );
  signal sif_fsm_b               : std_logic_vector( DATA_WIDTH-1 downto 0 );
  signal sif_fsm_c               : std_logic_vector( DATA_WIDTH-1 downto 0 );
  signal sif_fsm_s_0             : std_logic_vector(DATA_WIDTH-1 downto 0);
  signal sif_fsm_s_1             : std_logic_vector(DATA_WIDTH-1 downto 0);
  signal sif_fsm_s_2             : std_logic_vector(DATA_WIDTH-1 downto 0);
  signal sif_fsm_s_3             : std_logic_vector(DATA_WIDTH-1 downto 0);
  signal sif_fsm_x_speed         : std_logic_vector(DATA_WIDTH-1 downto 0);
  signal sif_fsm_y_speed         : std_logic_vector(DATA_WIDTH-1 downto 0);
  signal sif_fsm_rdy             : std_logic;
      
  -- PEU
  signal sig_peu_fsm_a               : std_logic_vector( DATA_WIDTH-1 downto 0 );
  signal sig_peu_fsm_b               : std_logic_vector( DATA_WIDTH-1 downto 0 );
  signal sig_peu_fsm_c               : std_logic_vector( DATA_WIDTH-1 downto 0 );
  signal sig_peu_fsm_x               : std_logic_vector( DATA_WIDTH-1 downto 0 );
  signal sig_peu_fsm_y               : std_logic_vector( DATA_WIDTH-1 downto 0 );
  signal sig_peu_fsm_not_cyc_write   : std_logic;
  --signal sig_peu_fsm_not_cyc_read  : std_logic;
  
  signal sig_peu_a                  : std_logic_vector(DATA_WIDTH-1 downto 0);
  signal sig_peu_b                  : std_logic_vector(DATA_WIDTH-1 downto 0);
  signal sig_peu_c                  : std_logic_vector(DATA_WIDTH-1 downto 0);
  signal sig_peu_x                  : std_logic_vector(DATA_WIDTH-1 downto 0);
  signal sig_peu_y                  : std_logic_vector(DATA_WIDTH-1 downto 0);
  signal sig_peu_act                : std_logic;
  signal sig_peu_vld                : std_logic;

  
  -- BDU
  signal sig_bdu_fsm_s_0             : std_logic_vector( DATA_WIDTH-1 downto 0 );
  signal sig_bdu_fsm_s_1             : std_logic_vector( DATA_WIDTH-1 downto 0 );
  signal sig_bdu_fsm_s_2             : std_logic_vector( DATA_WIDTH-1 downto 0 );
  signal sig_bdu_fsm_s_3             : std_logic_vector( DATA_WIDTH-1 downto 0 );
  signal sig_bdu_fsm_bar             : std_logic_vector( DATA_WIDTH-1 downto 0 );
  signal sig_bdu_fsm_not_cyc_write   : std_logic;
  --signal sig_bdu_fsm_not_cyc_read  : std_logic
  
  signal sig_bdu_s_0                : std_logic_vector(DATA_WIDTH-1 downto 0);
  signal sig_bdu_s_1                : std_logic_vector(DATA_WIDTH-1 downto 0);
  signal sig_bdu_s_2                : std_logic_vector(DATA_WIDTH-1 downto 0);
  signal sig_bdu_s_3                : std_logic_vector(DATA_WIDTH-1 downto 0);
  signal sig_bdu_bar                : std_logic_vector(DATA_WIDTH-1 downto 0);
  signal sig_bdu_act                : std_logic;
  signal sig_bdu_vld                : std_logic;
  
  -- MU
  signal sig_mu_act              : std_logic;
  signal sig_mu_vld              : std_logic;
  signal sig_mu_x                : std_logic_vector( DATA_WIDTH-1 downto 0 );
  signal sig_mu_y                : std_logic_vector( DATA_WIDTH-1 downto 0 );
  signal sig_mu_bar              : std_logic_vector( DATA_WIDTH-1 downto 0 );
  
  signal sig_mu_fsm_act          : std_logic;
  signal sig_mu_fsm_vld          : std_logic;   
  signal sig_mu_fsm_x            : std_logic_vector(DATA_WIDTH-1 downto 0);
  signal sig_mu_fsm_y            : std_logic_vector(DATA_WIDTH-1 downto 0);
  signal sig_mu_fsm_bar          : std_logic_vector(3 downto 0);
  
  -- MMU
  signal sig_addr_x_a            : std_logic_vector(DATA_WIDTH-1 downto 0);
  signal sig_addr_y_a            : std_logic_vector(DATA_WIDTH-1 downto 0);
  signal sig_rd_wr_a             : std_logic;
  signal sig_data_out_a          : std_logic_vector(3 downto 0);
  signal sig_data_in_a           : std_logic_vector(3 downto 0);
  signal sig_act_a               : std_logic;
  signal sig_vld_a               : std_logic;
  
  -- PFU
  signal sig_pfu_x_wb            : std_logic_vector(DATA_WIDTH-1 downto 0);
  signal sig_pfu_y_wb            : std_logic_vector(DATA_WIDTH-1 downto 0);
  signal sig_pfu_act_wb          : std_logic;
  signal sig_pfu_vld_wb          : std_logic;
  signal sig_pfu_x_ecu           : std_logic_vector(DATA_WIDTH-1 downto 0);
  signal sig_pfu_y_ecu           : std_logic_vector(DATA_WIDTH-1 downto 0);
  signal sig_pfu_act_ecu         : std_logic;
  signal sig_pfu_vld_ecu         : std_logic;
  signal sig_pfu_x_mmu           : std_logic_vector(DATA_WIDTH-1 downto 0);
  signal sig_pfu_y_mmu           : std_logic_vector(DATA_WIDTH-1 downto 0);
  signal sig_pfu_act_mmu         : std_logic;
  signal sig_pfu_vld_mmu         : std_logic;
  signal sig_pfu_rd_wr_mmu       : std_logic;
  signal sig_pfu_data_in_mmu     : std_logic_vector(3 downto 0);
   
  -- --- ChipScope modules: ----------------------------- 
   signal cs_control : std_logic_vector(35 downto 0); -- icon

    component chipscope_ila 
	 port (
       CONTROL: inout std_logic_vector(35 downto 0);
       CLK: in std_logic;
       TRIG0: in std_logic_vector(56 downto 0)
		 );
	  end component;
	
	  component chipscope_icon  
		port (
		  CONTROL0 : inout STD_LOGIC_VECTOR (35 downto 0 ) 
	  );
	  end component;
  -- ----------------------------------------------------------------

begin
  -- ChipScope modules: ----------------------
	  ila: chipscope_ila 
			port map(
				CONTROL => cs_control,
				CLK => CLK,
				TRIG0(0) => DATA_VLD, 									-- 0
				TRIG0(8 downto 1)   => S_0(ADDR_WIDTH-1 downto 0), 		-- 1 - 8
				TRIG0(12 downto 9)  => sif_fsm_s_0(3 downto 0),			-- 9 - 12
				TRIG0(20 downto 13) => S_1(ADDR_WIDTH-1 downto 0), 		-- 13 - 20
				TRIG0(24 downto 21) => sif_fsm_s_1(3 downto 0),			-- 21 - 24

				TRIG0(30 downto 25) => DIST_A(ADDR_WIDTH-2 downto 0),	-- 25 - 30 
				TRIG0(38 downto 31) => sif_fsm_a(ADDR_WIDTH-1 downto 0),-- 31 - 38
				TRIG0(46 downto 39) => DIST_B(ADDR_WIDTH-1 downto 0),	-- 39 - 46
				TRIG0(53 downto 47) => sif_fsm_b(ADDR_WIDTH-1 downto 0),-- 47 - 53
				TRIG0(54) => RST,										-- 54
				TRIG0(55) => sif_fsm_act,								-- 55
				TRIG0(56) => sif_fsm_vld								-- 56
				
			);
	
	  icon: chipscope_icon 
			port map(
				CONTROL0 => cs_control
			);
   -- ---------------------------------------

  intercon: entity work.intercon   
  port map(
      -- wishbone master port
      sif_dat_i => sif_dat_i,
      sif_ack_i => sif_ack_i,
      sif_dat_o => sif_dat_o,
      sif_we_o  => sif_we_o,
      sif_sel_o => sif_sel_o,
      sif_adr_o => sif_adr_o,
      sif_cyc_o => sif_cyc_o,
      sif_stb_o => sif_stb_o,
      
      mu_dat_i => mu_dat_i,
      mu_ack_i => mu_ack_i,
      mu_dat_o => mu_dat_o,
      mu_we_o  => mu_we_o,
      mu_sel_o => mu_sel_o,
      mu_adr_o => mu_adr_o,
      mu_cyc_o => mu_cyc_o,
      mu_stb_o => mu_stb_o,
      
      pfu_dat_i => pfu_dat_i,
      pfu_ack_i => pfu_ack_i,
      pfu_dat_o => pfu_dat_o,
      pfu_we_o  => pfu_we_o,
      pfu_sel_o => pfu_sel_o,
      pfu_adr_o => pfu_adr_o,
      pfu_cyc_o => pfu_cyc_o,
      pfu_stb_o => pfu_stb_o,
      -- wishbone slave port
      peu_dat_o => peu_dat_o,
      peu_ack_o => peu_ack_o,
      peu_dat_i => peu_dat_i,
      peu_we_i  => peu_we_i,
      peu_sel_i => peu_sel_i,
      peu_adr_i => peu_adr_i,
      peu_cyc_i => peu_cyc_i,
      peu_stb_i => peu_stb_i,
      
      bdu_dat_o => bdu_dat_o,
      bdu_ack_o => bdu_ack_o,
      bdu_dat_i => bdu_dat_i,
      bdu_we_i  => bdu_we_i,
      bdu_sel_i => bdu_sel_i,
      bdu_adr_i => bdu_adr_i,
      bdu_cyc_i => bdu_cyc_i,
      bdu_stb_i => bdu_stb_i,
      -- clock and reset
      clk            => CLK,
      reset          => RST

  ); 
  
  sif_wb : entity work.wb_sif_write_master_ent  
  port map(
      CLK            => CLK,
      RST            => RST,
      -- Wishbone signals
      ACK_I          => sif_ack_i,
      DAT_I          => sif_dat_i,
      DAT_O          => sif_dat_o,
      STB_O          => sif_stb_o,
      ADR_O          => sif_adr_o,
      CYC_O          => sif_cyc_o,
      WE_O           => sif_we_o,
      -- Output port (non-WISHBONE signals)
      ACT            => sif_fsm_act,
      VLD            => sif_fsm_vld,
      A              => sif_fsm_a,
      B              => sif_fsm_b,
      C              => sif_fsm_c,
      S_0            => sif_fsm_s_0,
      S_1            => sif_fsm_s_1,
      S_2            => sif_fsm_s_2,
      S_3            => sif_fsm_s_3
  );
  
  mu_wb : entity work.wb_mu_read_master_ent  
  port map(
      CLK            => CLK,
      RST            => RST,
      -- Wishbone signals
      ACK_I          => mu_ack_i,
      DAT_I          => mu_dat_i,
      DAT_O          => mu_dat_o,
      STB_O          => mu_stb_o,
      ADR_O          => mu_adr_o,
      CYC_O          => mu_cyc_o,
      WE_O           => mu_we_o,
      -- Output port (non-WISHBONE signals)
      ACT            => sig_mu_act,
      VLD            => sig_mu_vld,
      X              => sig_mu_x,
      Y              => sig_mu_y,
      BAR            => sig_mu_bar
  );
  
  pfu_wb : entity work.wb_pfu_read_master_ent  
  port map(
      CLK            => CLK,
      RST            => RST,
      -- Wishbone signals
      ACK_I          => pfu_ack_i,
      DAT_I          => pfu_dat_i,
      DAT_O          => pfu_dat_o,
      STB_O          => pfu_stb_o,
      ADR_O          => pfu_adr_o,
      CYC_O          => pfu_cyc_o,
      WE_O           => pfu_we_o,
      -- Output port (non-WISHBONE signals)
      -- =============================================================
      -- !!!!!! TADY PRIPOJIT PFU !!!!!
      -- =============================================================
      ACT            => sig_pfu_act_wb, --PFU_ACT,
      VLD            => sig_pfu_vld_wb, --PFU_VLD,
      X              => sig_pfu_x_wb, --PFU_X,
      Y              => sig_pfu_y_wb --PFU_Y
  );
  
  peu_wb : entity work.wb_peu_slave_ent   
  port map(
      CLK            => CLK,
      RST            => RST,
      -- Wishbone signals
      ACK_O          => peu_ack_o,
      DAT_I          => peu_dat_i,
      DAT_O          => peu_dat_o,
      STB_I          => peu_stb_i,
      ADR_I          => peu_adr_i,
      CYC_I          => peu_cyc_i,
      WE_I           => peu_we_i,
      -- Output port (non-WISHBONE signals)
      A              => sig_peu_fsm_a,
      B              => sig_peu_fsm_b,
      C              => sig_peu_fsm_c,
      X              => sig_peu_fsm_x,
      Y              => sig_peu_fsm_y,
      NOT_CYC_WRITE  => sig_peu_fsm_not_cyc_write
      --NOT_CYC_READ   => open--sig_peu_fsm_not_cyc_read
  );
  
  bdu_wb : entity work.wb_bdu_slave_ent   
  port map(
      CLK            => CLK,
      RST            => RST,
      -- Wishbone signals
      ACK_O          => bdu_ack_o,
      DAT_I          => bdu_dat_i,
      DAT_O          => bdu_dat_o,
      STB_I          => bdu_stb_i,
      ADR_I          => bdu_adr_i,
      CYC_I          => bdu_cyc_i,
      WE_I           => bdu_we_i,
      -- Output port (non-WISHBONE signals):
      S_0            => sig_bdu_fsm_s_0,
      S_1            => sig_bdu_fsm_s_1,
      S_2            => sig_bdu_fsm_s_2,
      S_3            => sig_bdu_fsm_s_3,
      BAR            => sig_bdu_fsm_bar,
      NOT_CYC_WRITE  => sig_bdu_fsm_not_cyc_write
      --NOT_CYC_READ   => open --bdu_fsm_not_cyc_read
  );
  
  sif_fsm : entity work.SIF_FSM_ENT
   port map(
      CLK            => CLK,
      RST            => RST,  
      -- ----------------- INPUT INTERFACE ----------------------- 
      A_IN           => DIST_A,
      B_IN           => DIST_B,
      C_IN           => DIST_C,
      S_0_IN         => S_0,
      S_1_IN         => S_1,
      S_2_IN         => S_2,
      S_3_IN         => S_3,
      DATA_VLD       => DATA_VLD, 
      X_SPEED        => X_SPEED,
      Y_SPEED        => Y_SPEED,
      -- ----------------- OUTPUT INTERFACE ----------------------
      A_OUT          => sif_fsm_a,
      B_OUT          => sif_fsm_b,
      C_OUT          => sif_fsm_c,
      S_0_OUT        => sif_fsm_s_0,
      S_1_OUT        => sif_fsm_s_1,
      S_2_OUT        => sif_fsm_s_2,
      S_3_OUT        => sif_fsm_s_3,
      ACT            => sif_fsm_act,
      VLD            => sif_fsm_vld,
	  RDY		     => sif_fsm_rdy,
      X_SPEED_IN     => sif_fsm_x_speed,
      Y_SPEED_IN     => sif_fsm_y_speed
   ); 
   
   peu_fsm : entity work.peu_fsm_ent
   port map(
      CLK            => CLK,
      RST            => RST,   
      -- ----------------- INPUT/OUTPUT DATA ---------------------  
      A              => sig_peu_fsm_a,
      B              => sig_peu_fsm_b,
      C              => sig_peu_fsm_c,
      NOT_CYC_WRITE  => sig_peu_fsm_not_cyc_write,             
      X              => sig_peu_fsm_x,
      Y              => sig_peu_fsm_y,
      -- ----------------- PEU DATA ---------------------------
      X_PEU          => sig_peu_x,
      Y_PEU          => sig_peu_y,     
      A_PEU          => sig_peu_a, 
      B_PEU          => sig_peu_b, 
      C_PEU          => sig_peu_c,    
      ACT_PEU        => sig_peu_act,
      VLD_PEU        => sig_peu_vld
   );  

   peu : entity work.peu_ent
   generic map(
      DATA_WIDTH  => DATA_WIDTH
   )   
   port map(
      CLK            => CLK,
      RST            => RST,
      -- input interface
      DIST_A         => sig_peu_a, 
      DIST_B         => sig_peu_b, 
      DIST_C         => sig_peu_c,         
      -- output interface      
      X              => sig_peu_x,
      Y              => sig_peu_y,    
      -- control interface
      ACT            => sig_peu_act,
      VLD            => sig_peu_vld
   );   

   bdu_fsm : entity work.bdu_fsm_ent
   port map
   (
      CLK            => CLK,
      RST            => RST, 
      
      -- ----------------- INPUT/OUTPUT DATA --------------------- 
      S_0            => sig_bdu_fsm_s_0,
      S_1            => sig_bdu_fsm_s_1,
      S_2            => sig_bdu_fsm_s_2,
      S_3            => sig_bdu_fsm_s_3,
      NOT_CYC_WRITE  => sig_bdu_fsm_not_cyc_write,
      BAR            => sig_bdu_fsm_bar,
      -- ----------------- BDU DATA ------------------------------ 
      S_0_BDU        => sig_bdu_s_0,
      S_1_BDU        => sig_bdu_s_1,
      S_2_BDU        => sig_bdu_s_2, 
      S_3_BDU        => sig_bdu_s_3,  
      BAR_BDU        => sig_bdu_bar,
      ACT_DBU        => sig_bdu_act,
      VLD_BDU        => sig_bdu_vld
   );
   
   bdu : entity work.bdu_ent
   generic map(
      DATA_WIDTH  => DATA_WIDTH
   )   
   port map(
      CLK            => CLK,
      RST            => RST,
      
      -- input interface
      S_0            => sig_bdu_s_0, 
      S_1            => sig_bdu_s_1, 
      S_2            => sig_bdu_s_2, 
      S_3            => sig_bdu_s_3, 
           
      -- output interface      
      OUTPUT         => sig_bdu_bar,
      
      -- control interface
      ACT            => sig_bdu_act,
      VLD            => sig_bdu_vld
   );
   
   mu_fsm : entity work.mu_fsm_ent
   port map
   (
      CLK            => CLK,
      RST            => RST, 
      
      -- ----------------- INPUT INTERFACE ----------------------- 
      WB_X           => sig_mu_x,
      WB_Y           => sig_mu_y,
      WB_BAR         => sig_mu_bar,
      WB_ACT         => sig_mu_act,
      WB_VLD         => sig_mu_vld,
	  RDY		     => sif_fsm_rdy,
              
      -- ----------------- OUTPUT INTERFACE ----------------------
      MU_X           => sig_mu_fsm_x,
      MU_Y           => sig_mu_fsm_y,
      MU_BAR         => sig_mu_fsm_bar,
      MU_ACT         => sig_mu_fsm_act,
      MU_VLD         => sig_mu_fsm_vld
   );
   
   map_unit : entity work.MU_ENT
   generic map(
      DATA_WIDTH  => DATA_WIDTH
   )   
   port map(
      CLK            => CLK,
      RST            => RST, 
      -- ----------------- INPUT/OUTPUT INTERFACE ----------------------------------
      ACT            => sig_mu_fsm_act,
      X              => sig_mu_fsm_x, 
      Y              => sig_mu_fsm_y,
      BARRIER        => sig_mu_fsm_bar,
      VLD            => sig_mu_fsm_vld,
      -- ----------------- MEMORY INTERFACE ---------------------------------   
      ADDR_X         => sig_addr_x_a,
      ADDR_Y         => sig_addr_y_a,
      RD_WR          => sig_rd_wr_a,
      DATA_OUT       => sig_data_in_a,
      DATA_IN        => sig_data_out_a,
      MEM_ACT        => sig_act_a,
      MEM_VLD        => sig_vld_a
   );
   
   map_memory_unit : entity work.MMU_ENT
   generic map(
      DATA_WIDTH  => DATA_WIDTH
   )   
   port map(
      -- ----------------- PORT A ---------------------------
      CLK_A             => CLK,
      RST_A             => RST,       
      ADDR_X_A          => sig_addr_x_a,
      ADDR_Y_A          => sig_addr_y_a,
      RD_WR_A           => sig_rd_wr_a,
      DATA_OUT_A        => sig_data_out_a,
      DATA_IN_A         => sig_data_in_a,
      ACT_A             => sig_act_a,
      VLD_A             => sig_vld_a,
      
      -- ----------------- PORT B ---------------------------
      -- =============================================================
      -- !!!!!! TADY PRIPOJIT PFU !!!!!
      -- =============================================================
      CLK_B             => CLK,
      RST_B             => RST,        
      ADDR_X_B          => sig_pfu_x_mmu, --ADDR_X,
      ADDR_Y_B          => sig_pfu_y_mmu, -- ADDR_Y,
      RD_WR_B           => sig_pfu_rd_wr_mmu, --'0',
      DATA_OUT_B        => sig_pfu_data_in_mmu, --DATA_OUT,
      DATA_IN_B         => "0000",--"0000",
      ACT_B             => sig_pfu_act_mmu, --'1',
      VLD_B             => sig_pfu_vld_mmu
   );
   
   engine_control_unit : entity work.ECU_ENT
   generic map(
      DATA_WIDTH  => DATA_WIDTH,
      QUEUE_SIZE  => QUEUE_SIZE
   ) 
   port map(
      CLK            => CLK,
      RST            => RST,
      -- ----------------- FIFO INPUT --------------------------------
      -- =============================================================
      -- !!!!!! TADY PRIPOJIT VYSTUP Z PFU !!!!!
      -- =============================================================
      X_IN           => sig_pfu_x_ecu, --"0000000000000000",
      Y_IN           => sig_pfu_y_ecu, --"0000000000000000",
      
      ACT_IN         => sig_pfu_act_ecu,--'0',
      VLD_IN         => sig_pfu_vld_ecu, --open,
      -- ----------------- MOVE OUTPUT -------------------------
      X_SPEED        => sif_fsm_x_speed,
      Y_SPEED        => sif_fsm_y_speed
   );
   
   path_finding_unit : entity work.PFU_ENT
   generic map(
      DATA_WIDTH  => DATA_WIDTH
   )   
   port map(
      CLK            => CLK,
      RST            => RST,
	  RDY		     => sif_fsm_rdy,
      
      -- ----------------- PFU_WB INTERFACE --------------------- 
      X_WB           => sig_pfu_x_wb,
      Y_WB           => sig_pfu_y_wb,
      ACT_WB         => sig_pfu_act_wb,
      VLD_WB         => sig_pfu_vld_wb,
      
      -- ----------------- ECU INTERFACE ------------------------ 
      X_ECU          => sig_pfu_x_ecu,
      Y_ECU          => sig_pfu_y_ecu,
      ACT_ECU        => sig_pfu_act_ecu,
      VLD_ECU        => sig_pfu_vld_ecu,
      
      -- ----------------- MMU INTERFACE ------------------------ 
      X_MMU          => sig_pfu_x_mmu,
      Y_MMU          => sig_pfu_y_mmu,
      ACT_MMU        => sig_pfu_act_mmu,
      VLD_MMU        => sig_pfu_vld_mmu,
      RD_WR_MMU      => sig_pfu_rd_wr_mmu,
      DATA_IN_MMU    => sig_pfu_data_in_mmu
   );  
end architecture;