-----------------------------------------------------------------------------
-- Project Name: Robot unit
-- File Name:    sif_fsm.vhd
-- Description:  Controll getting data from interface and send data do bus
-- Author:       Jakub Podivinsky <xpodiv01@stud.fit.vutbr.cz> 
-- Date:         28.1.2013 
-- --------------------------------------------------------------------------

library ieee;
use ieee.std_logic_1164.all;
use ieee.numeric_std.all;
use ieee.std_logic_signed.all;
-- ==========================================================================
--                              ENTITY DECLARATION
-- ==========================================================================
entity SIF_FSM_ENT is
   generic
   (
      DATA_WIDTH  : integer := 16      
   );
   
   port
   (
      CLK            : in  std_logic;
      RST            : in  std_logic; 
      
      -- ----------------- INPUT INTERFACE ----------------------- 
      A_IN           : in std_logic_vector(DATA_WIDTH-1 downto 0);
      B_IN           : in std_logic_vector(DATA_WIDTH-1 downto 0);
      C_IN           : in std_logic_vector(DATA_WIDTH-1 downto 0);
      S_0_IN         : in std_logic_vector(DATA_WIDTH-1 downto 0);
      S_1_IN         : in std_logic_vector(DATA_WIDTH-1 downto 0);
      S_2_IN         : in std_logic_vector(DATA_WIDTH-1 downto 0);
      S_3_IN         : in std_logic_vector(DATA_WIDTH-1 downto 0);
      DATA_VLD       : in  std_logic; 
      
      X_SPEED        : out std_logic_vector(DATA_WIDTH-1 downto 0);
      Y_SPEED        : out std_logic_vector(DATA_WIDTH-1 downto 0);
              
      -- ----------------- OUTPUT INTERFACE ----------------------
      A_OUT          : out std_logic_vector(DATA_WIDTH-1 downto 0);
      B_OUT          : out std_logic_vector(DATA_WIDTH-1 downto 0);
      C_OUT          : out std_logic_vector(DATA_WIDTH-1 downto 0);
      S_0_OUT        : out std_logic_vector(DATA_WIDTH-1 downto 0);
      S_1_OUT        : out std_logic_vector(DATA_WIDTH-1 downto 0);
      S_2_OUT        : out std_logic_vector(DATA_WIDTH-1 downto 0);
      S_3_OUT        : out std_logic_vector(DATA_WIDTH-1 downto 0);
      ACT            : out  std_logic;
      VLD            : in  std_logic;
      
      X_SPEED_IN     : in std_logic_vector(DATA_WIDTH-1 downto 0);
      Y_SPEED_IN     : in std_logic_vector(DATA_WIDTH-1 downto 0)
 
   );   
end entity;

-- ==========================================================================
--                           ARCHITECTURE DESCRIPTION
-- ==========================================================================
architecture SIF_FSM_ARCH of SIF_FSM_ENT is

-- ==========================================================================
--                                      TYPES
-- ==========================================================================
type state_type is (init_state, send_state); --stavy FSM

-- ==========================================================================
--                                    CONSTANTS
-- ==========================================================================
constant BOX_SIZE   : std_logic_vector(DATA_WIDTH-1 downto 0) := "0000000000001000";
-- ==========================================================================
--                                     SIGNALS
-- ==========================================================================
-- FSM signals 
signal state_reg, state_next: state_type;

-- data signals
signal reg_A                  : std_logic_vector(DATA_WIDTH-1 downto 0);
signal reg_B                  : std_logic_vector(DATA_WIDTH-1 downto 0);
signal reg_C                  : std_logic_vector(DATA_WIDTH-1 downto 0);
signal reg_S_0                : std_logic_vector(DATA_WIDTH-1 downto 0);
signal reg_S_1                : std_logic_vector(DATA_WIDTH-1 downto 0);
signal reg_S_2                : std_logic_vector(DATA_WIDTH-1 downto 0);
signal reg_S_3                : std_logic_vector(DATA_WIDTH-1 downto 0);

-- control signals
signal sig_we                 : std_logic;
signal sig_change_val         : std_logic;

signal sig_change_val_s_0     : std_logic;
signal sig_change_val_s_1     : std_logic;
signal sig_change_val_s_2     : std_logic;
signal sig_change_val_s_3     : std_logic;

begin
   X_SPEED <= X_SPEED_IN;
   Y_SPEED <= Y_SPEED_IN;


   sig_change_val_s_0 <= '1' when ((abs(reg_S_0 - S_0_IN) >= BOX_SIZE)) else '0';
   sig_change_val_s_1 <= '1' when ((abs(reg_S_1 - S_1_IN) >= BOX_SIZE)) else '0';
   sig_change_val_s_2 <= '1' when ((abs(reg_S_2 - S_2_IN) >= BOX_SIZE)) else '0';
   sig_change_val_s_3 <= '1' when ((abs(reg_S_3 - S_3_IN) >= BOX_SIZE)) else '0';
   
   -- detect event on input data => enable evaluating
   change_register : process (CLK, reg_S_0, reg_S_1, reg_S_2, reg_S_3, S_0_IN, S_1_IN, S_2_IN, S_3_IN, VLD)
   begin
      if( rising_edge( CLK ) ) then
         if (RST = '1') then
            sig_change_val <= '1';
         --elsif ((reg_S_0 /= S_0_IN and reg_S_1 /= S_1_IN) or (reg_S_2 /= S_2_IN and reg_S_3 /= S_3_IN)) then
         elsif (((sig_change_val_s_0 and sig_change_val_s_1) or (sig_change_val_s_2 and sig_change_val_s_3)) = '1') then
            sig_change_val <= '1';
         elsif (VLD = '1') then
            sig_change_val <= '0';
         end if;
      end if;
   end process;
   
   
   -- write to input registers A,B,C
   input_registers : process (CLK, RST, S_0_IN, S_1_IN, S_2_IN, S_3_IN, A_IN, B_IN, C_IN, sig_we)
   begin
      if( rising_edge( CLK ) ) then
         if (RST = '1') then
            --reg_A <= (others => '0');
            --reg_B <= (others => '0');
            --reg_C <= (others => '0');
            --reg_S_0 <= (others => '0');
            --reg_S_1 <= (others => '0');
            --reg_S_2 <= (others => '0');
            --reg_S_3 <= (others => '0');
         elsif (sig_we = '1') then
            reg_A <= A_IN;
            reg_B <= B_IN;
            reg_C <= C_IN;
            reg_S_0 <= S_0_IN;
            reg_S_1 <= S_1_IN;
            reg_S_2 <= S_2_IN;
            reg_S_3 <= S_3_IN;
         end if;
      end if;
   end process;
   
   A_OUT <= reg_A;
   B_OUT <= reg_B;
   C_OUT <= reg_C;
   S_0_OUT <= reg_S_0;
   S_1_OUT <= reg_S_1;
   S_2_OUT <= reg_S_2;
   S_3_OUT <= reg_S_3;
   
-- ------------------------- FSM --------------------------------------------   
   -- state logic
   fsm_state_logic : process (CLK)
   begin
     if (rising_edge(CLK)) then
        if (RST = '1') then
          state_reg <= init_state;
        else
          state_reg <= state_next;
        end if;   
     end if;   
   end process;
   
   -- next state logic
   fsm_next_state_logic : process (state_reg, DATA_VLD, VLD, sig_change_val)
   begin
     state_next <= state_reg;  
     
     case state_reg is
        when init_state =>
          if (DATA_VLD = '1' and sig_change_val = '1') then 
             state_next <= send_state;
             ACT <= '0';
             sig_we <= '1';
          else 
             state_next <= init_state;
             ACT <= '0';
             sig_we <= '0';
          end if;          
          
        when send_state => 
          if (VLD = '1') then 
             state_next <= init_state;
             ACT <= '1';
             sig_we <= '0';
          else 
             state_next <= send_state;
             ACT <= '1';
             sig_we <= '0';
          end if;
      end case;      
   end process;  
end architecture;
