-----------------------------------------------------------------------------
-- Project Name: ROBOT
-- File Name:    top_level.vhd
-- Description:  Propojuje vysilac(master), Wishbone sbernici a prijimac(slave). 
-- Author:       JP
-- Date:         9.11.2012 
-- --------------------------------------------------------------------------

library ieee;
use ieee.std_logic_1164.all;

entity TOP_LEVEL_ENT is
   generic
   (
      O_DATA_WIDTH   : integer  := 8;   
      ADDR_WIDTH     : natural  := 4; -- 3 would be enough (7 addresses)
      I_DATA_WIDTH   : integer  := 16 
   );
   port(
      CLK      : in std_logic;
      RST      : in std_logic;

      DATA     : in std_logic_vector(O_DATA_WIDTH-1 downto 0);
      SEL      : in std_logic_vector(ADDR_WIDTH-1 downto 0);
		VLD		: in std_logic;
      
      X_SPEED  : out std_logic_vector(O_DATA_WIDTH-1 downto 0);
      Y_SPEED  : out std_logic_vector(O_DATA_WIDTH-1 downto 0)
  );
end entity;

architecture TOP_LEVEL_ARCH of TOP_LEVEL_ENT is

   signal sig_dist_a          : std_logic_vector(I_DATA_WIDTH-1 downto 0);
   signal sig_dist_b          : std_logic_vector(I_DATA_WIDTH-1 downto 0);
   signal sig_dist_c          : std_logic_vector(I_DATA_WIDTH-1 downto 0);
   signal sig_s_0             : std_logic_vector(I_DATA_WIDTH-1 downto 0);
   signal sig_s_1             : std_logic_vector(I_DATA_WIDTH-1 downto 0);
   signal sig_s_2             : std_logic_vector(I_DATA_WIDTH-1 downto 0);
   signal sig_s_3             : std_logic_vector(I_DATA_WIDTH-1 downto 0);
   signal sig_data_vld        : std_logic; 
   signal sig_x_speed         : std_logic_vector(I_DATA_WIDTH - 1 downto 0);
   signal sig_y_speed         : std_logic_vector(I_DATA_WIDTH - 1 downto 0);

  -- --- ChipScope modules: ----------------------------- 
   --signal cs_control : std_logic_vector(35 downto 0); -- icon

    --component chipscope_ila 
	-- port (
    --   CONTROL: inout std_logic_vector(35 downto 0);
    --   CLK: in std_logic;
    --   TRIG0: in std_logic_vector(56 downto 0)
	--	 );
	--  end component;
	--
	-- component chipscope_icon  
	--	port (
	--	  CONTROL0 : inout STD_LOGIC_VECTOR (35 downto 0 ) 
	--  );
	--  end component;
  -- ----------------------------------------------------------------

begin
  -- ChipScope modules: ----------------------
	--  ila: chipscope_ila 
	--		port map(
	--			CONTROL => cs_control,
	--			CLK => CLK,
	--			--TRIG0(7 downto 0)  => DATA,
	--			--TRIG0(11 downto 8) => SEL,
	--			--TRIG0(12)=> VLD,
	--			--TRIG0(13)=> VLD,
	--			--TRIG0(14)=> VLD,
	--			--TRIG0(15)=> VLD,
	--			TRIG0(7 downto 0)  => sig_s_0(O_DATA_WIDTH-1 downto 0),
	--			TRIG0(15 downto 8) => sig_s_1(O_DATA_WIDTH-1 downto 0),
	--			TRIG0(23 downto 16) => sig_s_2(O_DATA_WIDTH-1 downto 0),
	--			TRIG0(31 downto 24) => sig_s_3(O_DATA_WIDTH-1 downto 0),
	--			TRIG0(39 downto 32) => sig_dist_A(O_DATA_WIDTH-1 downto 0),
	--			TRIG0(47 downto 40) => sig_dist_B(O_DATA_WIDTH-1 downto 0),
	--			TRIG0(55 downto 48) => sig_dist_C(O_DATA_WIDTH-1 downto 0),
	--			TRIG0(56) => sig_data_vld
	--		);
	--
	--  icon: chipscope_icon 
	--		port map(
	--			CONTROL0 => cs_control
	--		);
   -- ---------------------------------------
 
   iface : entity work.INTERFACE_ENT
   generic map(
      DATA_WIDTH => 8,
      REG_IN	  => 7, -- number of senzoric signals    
      ADDR_WIDTH => 4  -- 3 would be enough (7 addresses)
   )
   port map(
      CLK         => CLK,
      RST         => RST,

      DATA        => DATA,
      SEL         => SEL,
		VLD		   => VLD,

      DA          => sig_dist_a(O_DATA_WIDTH-1 downto 0),
      DB          => sig_dist_b(O_DATA_WIDTH-1 downto 0),
      DC          => sig_dist_c(O_DATA_WIDTH-1 downto 0),
      DL          => sig_s_0(O_DATA_WIDTH-1 downto 0),
      DR          => sig_s_1(O_DATA_WIDTH-1 downto 0),
      DU          => sig_s_2(O_DATA_WIDTH-1 downto 0),
      DD          => sig_s_3(O_DATA_WIDTH-1 downto 0),
      RDY         => sig_data_vld
   );
   
   robot_controller : entity work.ROBOT_CONTROLLER_ENT
   generic map (
      DATA_WIDTH     =>  16,  
      ADDR_WIDTH     =>  8,
      OFFSET_WIDTH   =>  4,
      QUEUE_SIZE     =>  15      
   )
   port map(
      CLK            => CLK,
      RST            => RST,
      
      DIST_A         => sig_dist_a,
      DIST_B         => sig_dist_b,
      DIST_C         => sig_dist_c,
      S_0            => sig_s_0,
      S_1            => sig_s_1,
      S_2            => sig_s_2,
      S_3            => sig_s_3,
      DATA_VLD       => sig_data_vld,
      X_SPEED        => sig_x_speed,
      Y_SPEED        => sig_y_speed
		--PFU_ACT        => '1',
      --PFU_VLD        => open,
      --PFU_X          => open,
      --PFU_Y          => open,
      
      -- =================================================================
      -- PFU MEM
      -- =================================================================
      -- ----------------- PORT B ---------------------------       
      --ADDR_X         => "0000000000000000",
      --ADDR_Y         => "0000000000000000",
      --DATA_OUT       => open
   );
   
   X_SPEED <= sig_x_speed(O_DATA_WIDTH-1 downto 0);
   Y_SPEED <= sig_y_speed(O_DATA_WIDTH-1 downto 0);
   

end architecture;
